<?php
/**
 * Developer : MahdiY
 * Web Site  : MahdiY.IR
 * E-Mail    : M@hdiY.IR
 */

defined( 'ABSPATH' ) || exit;

class PWS_Cart {

	public static function get_weight(): float {

		$weight = floatval( PWS()->get_option( 'tools.package_weight', 500 ) );

		$cart_items = WC()->cart ? WC()->cart->get_cart() : [];

		foreach ( $cart_items as $cart_item ) {
			$weight += PWS_Product::get_weight( $cart_item['data'] ) * $cart_item['quantity'];
		}

		return floatval( apply_filters( 'pws_cart_weight', $weight ) );
	}

	/**
	 * @param string $shipping_class Shipping class slug
	 *
	 * @return bool
	 */
	public static function has_shipping_class( string $shipping_class ): bool {

		$cart_items = WC()->cart ? WC()->cart->get_cart() : [];

		foreach ( $cart_items as $cart_item ) {

			/** @var WC_Product $product */
			$product = $cart_item['data'];

			if ( $product->get_shipping_class() == $shipping_class ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * @param int $product_id
	 *
	 * @return bool
	 */
	public static function has_product( int $product_id ): bool {

		$cart_items = WC()->cart ? WC()->cart->get_cart() : [];

		foreach ( $cart_items as $cart_item ) {

			/** @var WC_Product $product */
			$product = $cart_item['data'];

			if ( in_array( $product_id, [ $product->get_id(), $product->get_parent_id() ] ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * @param int $category_id
	 *
	 * @return bool
	 */
	public static function has_category( int $category_id ): bool {

		$cart_items = WC()->cart ? WC()->cart->get_cart() : [];

		foreach ( $cart_items as $cart_item ) {

			/** @var WC_Product $product */
			$product = $cart_item['data'];

			$categories = $product->get_category_ids();

			foreach ( $categories as $category ) {
				$categories = array_merge( $categories, get_ancestors( $category, 'product_cat', 'taxonomy' ) );
			}

			if ( in_array( $category_id, $categories ) ) {
				return true;
			}
		}

		return false;
	}

}
