<?php
/**
 * Developer : MahdiY
 * Web Site  : MahdiY.IR
 * E-Mail    : M@hdiY.IR
 */

defined( 'ABSPATH' ) || exit;

if ( class_exists( 'PWS_Tapin_Method' ) ) {
	return;
} // Stop if the class already exists

/**
 * Class WC_Tapin_Method
 *
 * @author mahdiy
 *
 */
class PWS_Tapin_Method extends PWS_Shipping_Method {

	protected $method = null;

	public function init() {

		parent::init();

		$this->extra_cost = $this->get_option( 'extra_cost', 0 );
		$this->fixed_cost = $this->get_option( 'fixed_cost' );

		add_action( 'woocommerce_update_options_shipping_' . $this->id, [ $this, 'process_admin_options' ] );
	}

	public function init_form_fields() {

		$currency_symbol = get_woocommerce_currency_symbol();

		$this->instance_form_fields += [
			'extra_cost' => [
				'title'       => 'هزینه های اضافی',
				'type'        => 'text',
				'description' => 'هزینه های اضافی علاوه بر نرخ پستی را می توانید وارد نمائید، (مثل: هزینه های بسته بندی و ...) مبلغ ثابت را به ' . $currency_symbol . ' وارد نمائید',
				'default'     => 0,
				'desc_tip'    => true,
			],
			'fixed_cost' => [
				'title'       => 'هزینه ثابت',
				'type'        => 'text',
				'description' => "<b>توجه:</b>
								<ul>
									<li>1. برای محاسبه هزینه توسط فرمول تاپین/پست کتاب خالی بگذارید.</li>
									<li>2. صفر به معنی رایگان است. یعنی هزینه حمل و نقل برعهده فروشگاه شما است.</li>
									<li>3. در صورت تعیین هزینه ثابت حمل و نقل این قیمت دقیقا به مشتری نمایش داده می شود.</li>
									<li>4. این گزینه مناسب فروشگاه هایی است که وزن محصولات خود را وارد نکرده اند.</li>
								</ul>
								",
				'default'     => '',
			],
		];
	}

	public function is_available( $package = [] ): bool {

		if ( ! PWS_Tapin::is_enable() ) {
			return false;
		}

		$weight = PWS_Cart::get_weight();

		if ( $weight > 30000 ) {
			return false;
		}

		return parent::is_available( $package );
	}

	public function calculate_shipping( $package = [] ) {

		if ( $this->free_shipping( $package ) ) {
			return;
		}

		$options = PWS()->get_terms_option( $this->get_destination( $package ) );
		$options = array_column( $options, $this->method == 'pishtaz' ? 'forehand_cost' : 'custom_cost' );

		foreach ( $options as $option ) {
			if ( $option != '' ) {
				$this->add_rate_cost( $option, $package );

				return;
			}
		}

		if ( $this->fixed_cost !== '' ) {
			$this->add_rate_cost( $this->fixed_cost, $package );

			return;
		}

		$weight = PWS_Cart::get_weight();

		$price = 0;

		foreach ( WC()->cart->get_cart() as $cart_item ) {

			if ( $cart_item['data']->is_virtual() ) {
				continue;
			}

			$price += $cart_item['data']->get_price() * $cart_item['quantity'];
		}

		$destination = $package['destination'];

		$payment_method = WC()->session->get( 'chosen_payment_method' );

		$is_cod = $payment_method === 'cod';

		if ( get_woocommerce_currency() == 'IRT' ) {
			$price *= 10;
		}

		if ( get_woocommerce_currency() == 'IRHR' ) {
			$price *= 1000;
		}

		if ( get_woocommerce_currency() == 'IRHT' ) {
			$price *= 10000;
		}

		$shop = PWS_Tapin::shop();

		$data = [
			'price'         => intval( $price ),
			'weight'        => ceil( $weight ),
			'method'        => $this->method,
			'is_cod'        => $is_cod,
			'to_province'   => intval( $destination['state'] ),
			'from_province' => intval( $shop->province_code ?? 1 ),
			'to_city'       => intval( $destination['city'] ),
			'from_city'     => intval( $shop->city_code ?? 1 ),
		];

		$cost = $this->calculate_rates( $data );

		if ( $cost === false ) {
			return;
		}

		$shipping_total = $cost + ( $shop->total_price ?? 0 );

		if ( PWS()->get_option( 'tapin.roundup_price' ) ) {
			$shipping_total = ceil( $shipping_total / 1000 ) * 1000;
		}

		$shipping_total = PWS()->convert_currency( $shipping_total );

		$shipping_total += $this->extra_cost;

		$this->add_rate_cost( $shipping_total, $package );
	}

	public function calculate_rates( array $args ): int {

		$defaults = [
			'price'         => 50000,
			'weight'        => 100,
			'method'        => 'pishtaz',
			'is_cod'        => false,
			'to_province'   => 1,
			'to_city'       => 1,
			'from_province' => 1,
			'from_city'     => 1,
			'gateway'       => PWS()->get_option( 'tapin.gateway' ),
		];

		$args = wp_parse_args( $args, $defaults );

		$weight_indicator = 9999;

		switch ( true ) {
			case $args['weight'] <= 500:
				$weight_indicator = 500;
				break;
			case $args['weight'] > 500 && $args['weight'] <= 1000:
				$weight_indicator = 1000;
				break;
			case $args['weight'] > 1000 && $args['weight'] <= 2000:
				$weight_indicator = 2000;
				break;
			case $args['weight'] > 2000 && $args['weight'] <= 3000:
				$weight_indicator = 3000;
				break;
			case $args['weight'] > 3000 && $args['weight'] <= 4000:
				$weight_indicator = 4000;
				break;
			case $args['weight'] > 4000 && $args['weight'] <= 5000:
				$weight_indicator = 5000;
				break;
		}

		$checked_state = PWS()->check_states_beside( $args['from_province'], $args['to_province'] );

		$rates = $this->rates( $args['gateway'], $args['method'] );

		if ( $rates === false ) {
			return false;
		}

		// calculate
		if ( $weight_indicator != 9999 ) {
			$cost = $rates[ $weight_indicator ][ $checked_state ];
		} else {
			$cost = $rates[5000][ $checked_state ] + ( $rates[ $weight_indicator ] * ceil( ( $args['weight'] - 5000 ) / 1000 ) );
		}

		if ( in_array( $args['to_city'], [ 1, 31, 51, 81, 71, 91, 61 ] ) ) {
			$cost += $cost * 0.1;
		}

		// insurance
		if ( $args['price'] <= 8000000 ) {
			$cost += 8000 + 11000;
		} else {
			$cost += $args['price'] * 0.002 + 11000;
		}

		if ( $args['is_cod'] ) {

			$_rate  = 0.015;
			$_price = 5000;

			if ( $checked_state != 'in' ) {

				$_rate  = 0.03;
				$_price = 10000;

			}

			$cost += min( 2000000, $args['price'] ) * $_rate;

			$cost += ceil( ( $args['price'] - 2000000 ) / 2000000 ) * $_price;

		}

		$cost += $cost * 0.09;

		return $cost;
	}

	public function rates( $gateway, $method ) {

		$rates = [
			'tapin'      => [
				'pishtaz'   => [
					500  => [
						'in'     => 57500,
						'beside' => 78000,
						'out'    => 84000,
					],
					1000 => [
						'in'     => 74000,
						'beside' => 100000,
						'out'    => 112000,
					],
					2000 => [
						'in'     => 98000,
						'beside' => 127000,
						'out'    => 140000,
					],
					3000 => [
						'in'     => 123000,
						'beside' => 152000,
						'out'    => 165000,
					],
					4000 => [
						'in'     => 148000,
						'beside' => 177000,
						'out'    => 190000,
					],
					5000 => [
						'in'     => 173000,
						'beside' => 202000,
						'out'    => 215000,
					],
					9999 => 25000,
				],
				'sefareshi' => [
					500  => [
						'in'     => 36800,
						'beside' => 49000,
						'out'    => 53000,
					],
					1000 => [
						'in'     => 48300,
						'beside' => 67600,
						'out'    => 72800,
					],
					2000 => [
						'in'     => 69000,
						'beside' => 88000,
						'out'    => 95000,
					],
					3000 => [
						'in'     => 66700,
						'beside' => 108000,
						'out'    => 116000,
					],
					4000 => [
						'in'     => 66700,
						'beside' => 108000,
						'out'    => 116000,
					],
					5000 => [
						'in'     => 66700,
						'beside' => 108000,
						'out'    => 116000,
					],
					9999 => 10000,
				],
			],
			'posteketab' => [
				'pishtaz'   => [
					500  => [
						'in'     => 40250,
						'beside' => 54600,
						'out'    => 58800,
					],
					1000 => [
						'in'     => 51800,
						'beside' => 70000,
						'out'    => 78400,
					],
					2000 => [
						'in'     => 68600,
						'beside' => 88900,
						'out'    => 98000,
					],
					3000 => [
						'in'     => 86100,
						'beside' => 106400,
						'out'    => 115500,
					],
					4000 => [
						'in'     => 103600,
						'beside' => 123900,
						'out'    => 133000,
					],
					5000 => [
						'in'     => 121100,
						'beside' => 141400,
						'out'    => 150500,
					],
					9999 => 17500,
				],
				'sefareshi' => [
					500  => [
						'in'     => 27600,
						'beside' => 36750,
						'out'    => 39750,
					],
					1000 => [
						'in'     => 36225,
						'beside' => 50700,
						'out'    => 54600,
					],
					2000 => [
						'in'     => 51750,
						'beside' => 66000,
						'out'    => 71250,
					],
					3000 => [
						'in'     => 50025,
						'beside' => 81000,
						'out'    => 87000,
					],
					4000 => [
						'in'     => 50025,
						'beside' => 81000,
						'out'    => 87000,
					],
					5000 => [
						'in'     => 50025,
						'beside' => 81000,
						'out'    => 87000,
					],
					9999 => 7500,
				],
			],
		];

		return $rates[ $gateway ][ $method ] ?? false;
	}
}
